/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997
 *	Sleepycat Software.  All rights reserved.
 */

#include "config.h"

#ifndef lint
static const char sccsid[] = "@(#)cxx_log.cpp	10.3 (Sleepycat) 7/20/97";
#endif /* not lint */

#include "db_cxx.h"
#include "cxx_int.h"
#include <errno.h>

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            DBLog                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

DBLog::DBLog()
:   imp_(0)
{
}

DBLog::DBLog(const char *dir, int flags, int mode, DBApp &dbapp)
:   imp_(0)
{
    int err;
    DB_LOG *log = 0;

    if ((err = log_open(dir, flags, mode, unwrap(&dbapp), &log)) != 0) {
        DB_ERROR("DBLog::open", err);
    }
    imp_ = wrap(log);
}

int DBLog::close()
{
    int err;
    DB_LOG *log = unwrap(this);
    if ((err = log_close(log)) != 0) {
        DB_ERROR("DBLog::close", err);
        return err;
    }
    return 0;
}

int DBLog::Compare(DBSequenceNumber sn1, DBSequenceNumber sn2)
{
    return log_compare(unwrap(&sn1), unwrap(&sn2));
}

int DBLog::file(DBSequenceNumber sn, char *name_returned, int len)
{
    int err;
    DB_LOG *log = unwrap(this);
    if ((err = log_file(log, unwrap(&sn), name_returned, len)) != 0) {
        DB_ERROR("DBLog::file", err);
        return err;
    }
    return 0;
}

int DBLog::flush(DBSequenceNumber sn)
{
    int err;
    DB_LOG *log = unwrap(this);
    if ((err = log_flush(log, unwrap(&sn))) != 0) {
        DB_ERROR("DBLog::flush", err);
        return err;
    }
    return 0;
}

int DBLog::get(DBSequenceNumber sn, DBThunk *data_returned, int flags)
{
    int err;
    DB_LOG *log = unwrap(this);
    if ((err = log_get(log, unwrap(&sn), unwrap(data_returned), flags)) != 0) {
        DB_ERROR("DBLog::get", err);
        return err;
    }
    return 0;
}

int DBLog::put(DBSequenceNumber *sn_returned, const DBThunk &data, int flags)
{
    int err = 0;
    DB_LOG *log = unwrap(this);
    if ((err = log_put(log, unwrap(sn_returned),
                       unwrapConst(&data), flags)) != 0) {
        DB_ERROR("DBLog::put", err);
        return err;
    }
    return 0;
}

int DBLog::identify(DBTable &dbp, const char *name, u_int32_t *fid_returned)
{
    int err = 0;
    DB_LOG *log = unwrap(this);
    if (!dbp.isValid()) {
        DB_ERROR("DBLog::identify", EINVAL);
        return err;
    }
    DBTYPE type = (DBTYPE)dbp.kind();
    if ((err = log_register(log, unwrap(&dbp), name, type,
                            fid_returned)) != 0) {
        DB_ERROR("DBLog::identify", err);
        return err;
    }
    return 0;
}

int DBLog::unidentify(u_int32_t fid)
{
    int err;
    DB_LOG *log = unwrap(this);
    if ((err = log_unregister(log, fid)) != 0) {
        DB_ERROR("DBLog::unidentify", err);
        return err;
    }
    return 0;
}

int DBLog::isValid() const
{
    return (unwrapConst(this) != 0);
}
