/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997
 *	Sleepycat Software.  All rights reserved.
 */

#include "config.h"

#ifndef lint
static const char copyright[] =
"@(#) Copyright (c) 1997\n\
	Sleepycat Software Inc.  All rights reserved.\n";
static const char sccsid[] = "@(#)AccessExample.cpp	10.3 (Sleepycat) 7/21/97";
#endif

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#include <iostream.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#ifndef _MSC_VER
#include <unistd.h>
#endif
#endif

#include <db_cxx.h>

class AccessExample : public DBApp
{
public:
    void run();

    AccessExample(const char *home, DBAppOptions &options);

private:
    static const char FileName[];

    // no need for copy and assignment
    AccessExample(const AccessExample &);
    operator = (const AccessExample &);
};

static void usage();          // forward

int main(int argc, char *argv[])
{
    const char *home = 0;
    for (int i = 1; i < argc; ++i)
    {
        if (strcmp(argv[i], "-h") == 0)
        {
            home = argv[++i];
        }
        else
        {
            usage();
        }
    }

    try
    {
        DBAppOptions options;
        options.setErrorStream(&cerr);

        AccessExample app(home, options);
        app.run();
        return 0;
    }
    catch (DBException &dbe)
    {
        cerr << "AccessExample: " << dbe.what() << "\n";
        return 1;
    }
}

static void usage()
{
    cerr << "usage: AccessExample [-h home]\n";
    exit(1);
}

const char AccessExample::FileName[] = "access.db";

AccessExample::AccessExample(const char *home, DBAppOptions &options)
:   DBApp(home, options)
{
    setName("AccessExample");
}

void AccessExample::run()
{
    cout << DBVersion::GetVersionString() << "\n";
    cout << DBVersion::GetVersion().getMajor() << "\n";
    cout << DBVersion::GetVersion().getMinor() << "\n";
    cout << DBVersion::GetVersion().getPatch() << "\n";

    DBBTreeTable table;
    createBTreeTable(&table, FileName, 0644);

    //
    // Insert records into the database, where the key is the user
    // input and the data is the user input in reverse order.
    //
    DBThunk key;
    DBThunk data;

    char buf[1024];
    char rbuf[1024];

    for (;;) {
        cout << "input> ";
        cout.flush();

        cin.getline(buf, sizeof(buf));
        if (cin.eof())
            break;

        int len = strlen(buf);
        if (len <= 0)
            continue;

        char *t = rbuf;
        char *p = buf + len - 1;
        while (p >= buf)
            *t++ = *p--;
        *t = '\0';

        key.setData(buf, len+1);
        data.setData(rbuf, len+1);

        try
        {
            table.put(key, data);
        }
        catch (DBKeyExistsException &dbe)
        {
            cout << dbe.what() << "\n";
            cout << "Key " << buf << " already exists.\n";
        }
        cout << "\n";
    }

    // Acquire an iterator for the table.
    DBIterator iterator;
    table.getIterator(&iterator);

    // Walk through the table, printing the key/data pairs.
    for (; !iterator.noMore(); iterator.next())
    {
        iterator.getKey(&key);
        iterator.getValue(&data);

        if (key.getSize() <= 0 ||
            data.getSize() <= 0) {
            cerr << "error retrieving data\n.";
        }
        char *key_string = (char *)key.getData();
        char *data_string = (char *)data.getData();
        cout << key_string << " : " << data_string << "\n";
    }
}
