/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997
 *	Sleepycat Software.  All rights reserved.
 */

#include "config.h"

#ifndef lint
static const char copyright[] =
"@(#) Copyright (c) 1997\n\
	Sleepycat Software Inc.  All rights reserved.\n";
static const char sccsid[] = "@(#)AppinitExample.cpp	10.2 (Sleepycat) 7/20/97";
#endif

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#include <errno.h>
#include <iostream.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#endif

#include <db_cxx.h>

#define	DATABASE_HOME	"/home/database"

DB_ENV *db_setup(FILE *);

char *progname = "AppinitExample";			/* Program name. */

/*
 * An example of a program using DBApp to configure its DB
 * environment.
 */
void
init()
{
    DBAppOptions options;
    ostream &errstr = cerr;

    DBConfig config;

    //
    // All of the shared database files live in /home/database,
    // but data files are created in /database/files.
    //
    config.setDataDirectoryForCreation("/database/files");

    // Output errors to the application's log.
    options.setErrorStream(&errstr);
    options.setErrorPrefix(progname);

    // The application uses locking, memory pools and needs
    // underlying files created.
    //
    options.setCreateFiles(1);
    options.setUseLocking(1);
    options.setUseMemPools(1);

    //
    // We want to specify the shared memory buffer pool cachesize,
    // but everything else is the default.
    //
    options.setMemPoolCacheSize(64 * 1024);

    // Constructing the app initializes the database using
    // the specified options and configuration.
    // Alternatively, one can inherit from DBApp for similar
    // functionality.  See the other examples.
    //
    DBApp app(DATABASE_HOME, options, config);

    // Now app can be used as needed (see other examples)...
    //

}

int
main(int, char **)
{
    //
    // Note: it may be easiest to put all DB operations in a try block, as
    // seen here.  Alternatively, you can change the ErrorModel in
    // DBAppOptions so that exceptions are never thrown and check
    // error returns from all methods.
    //
    try
    {
        init();
        return 0;
    }
    catch (DBException &dbe)
    {
        cerr << "AccessExample: " << dbe.what() << "\n";
        return 1;
    }
}
